==> ex_open_sea-0.0.5.tar: lib/ex_open_sea/collections/index.ex <==
defmodule ExOpenSea.Collections.Index do
  @moduledoc """
  Return a list of all the collections supported and vetted by OpenSea. To include all collections
  relevant to a user (including non-whitelisted ones) include the `asset_owner` parameter.

  Each collection in the response has an attribute called `primary_asset_contracts` with info about
  the smart contracts belonging to that collection. For example, ERC-1155 contracts maybe have
  multiple collections all referencing the same contract, but many ERC-721 contracts may all
  belong to the same collection (dapp).

  You can also use this endpoint to find which dapps an account uses, and how many items they own
  in each in a single API call.

  https://docs.opensea.io/reference/retrieving-collections
  """

  alias ExOpenSea.Http

  @type api_key :: ExOpenSea.ApiKey.t()
  @type params :: %{
    optional(:asset_owner) => String.t(),
    optional(:offset) => non_neg_integer,
    optional(:limit) => non_neg_integer,
  }
  @type collection :: ExOpenSea.Collection.t()
  @type error_reason :: :parse_result_item | String.t()
  @type result :: {:ok, [collection]} | {:error, error_reason}

  @spec get(api_key) :: result
  @spec get(api_key, params) :: result
  def get(api_key, params \\ %{}) do
    "/api/v1/collections"
    |> Http.Request.for_path()
    |> Http.Request.with_query(params)
    |> Http.Request.with_auth(api_key)
    |> Http.Client.get()
    |> parse_response()
  end

  defp parse_response({:ok, %{"collections" => collections}}) do
    collections
    |> Enum.map(&Mapail.map_to_struct(&1, ExOpenSea.Collection))
    |> Enum.reduce(
      {:ok, []},
      fn
        {:ok, i}, {:ok, acc} -> {:ok, [i | acc]}
        _, _acc -> {:error, :parse_result_item}
      end
    )
  end

  defp parse_response({:error, response_reasons}) when is_map(response_reasons) do
    reasons = response_reasons
              |> Enum.reduce(
                [],
                fn {k, v}, acc ->
                  acc ++ [{k, v}]
                end
              )

    {:error, reasons}
  end

  defp parse_response({:error, _reason} = error) do
    error
  end
end

==> ex_open_sea-0.0.6.tar: lib/ex_open_sea/collections/index.ex <==
defmodule ExOpenSea.Collections.Index do
  @moduledoc """
  Return a list of all the collections supported and vetted by OpenSea. To include all collections
  relevant to a user (including non-whitelisted ones) include the `asset_owner` parameter.

  Each collection in the response has an attribute called `primary_asset_contracts` with info about
  the smart contracts belonging to that collection. For example, ERC-1155 contracts maybe have
  multiple collections all referencing the same contract, but many ERC-721 contracts may all
  belong to the same collection (dapp).

  You can also use this endpoint to find which dapps an account uses, and how many items they own
  in each in a single API call.

  https://docs.opensea.io/reference/retrieving-collections
  """

  alias ExOpenSea.Http

  @type params :: %{
    optional(:asset_owner) => String.t(),
    optional(:offset) => non_neg_integer,
    optional(:limit) => non_neg_integer,
  }
  @type collection :: ExOpenSea.Collection.t()
  @type error_reason :: :parse_result_item | String.t()
  @type result :: {:ok, [collection]} | {:error, error_reason}

  @spec get(params) :: result
  def get(params \\ %{}) do
    "/api/v1/collections"
    |> Http.Request.for_path()
    |> Http.Request.with_query(params)
    |> Http.Client.get()
    |> parse_response()
  end

  defp parse_response({:ok, %{"collections" => collections}}) do
    collections
    |> Enum.map(&Mapail.map_to_struct(&1, ExOpenSea.Collection))
    |> Enum.reduce(
      {:ok, []},
      fn
        {:ok, i}, {:ok, acc} -> {:ok, [i | acc]}
        _, _acc -> {:error, :parse_result_item}
      end
    )
  end

  defp parse_response({:error, response_reasons}) when is_map(response_reasons) do
    reasons = response_reasons
              |> Enum.reduce(
                [],
                fn {k, v}, acc ->
                  acc ++ [{k, v}]
                end
              )

    {:error, reasons}
  end

  defp parse_response({:error, _reason} = error) do
    error
  end
end
